#!/usr/bin/perl

# sendReport.pl - Perl Script to generate Introscope reports and send them as email attachments
# Version 0.1 - 24 October 2006
# Sylvain Gibassier - Wily Technology

use warnings;
use strict;
use Getopt::Long;
use Date::Manip;
use Net::SMTP;
use MIME::Lite;

##################################
#                               Constants                                   #
##################################
my %reportFormats=('html'=>'text/html','pdf'=>'application/pdf','xls'=>'application/x-excel','txt'=>'text/plain','xml'=>'text/xml','csv'=>'text/csv');
my $clwDateFormat="%Y-%m-%d %r";
my $clwJavaHeap=128;
my $smtpNormalSubject='Introscope\'s reports';
my $smtpNormalMessage = <<SMTP_NORMAL_MESSAGE
	Dear Colleague,
	
Please find attached the Introscope reports that you have requested.

	Best Regards,
	
		The Introscope Team

SMTP_NORMAL_MESSAGE
;

my $smtpErrorSubject='Error while generating Introscope reports';
my $smtpErrorMessage = <<SMTP_ERROR_MESSAGE
Unable to generate the Introscope reports. The following error occured:
SMTP_ERROR_MESSAGE
;

##################################
#        Parse and validate command line arguments              #
##################################

my $reportName='';
my $managementModule='';
my $smtpTo='';
my @smtpAddresses;
my $smtpErrorTo='';
my @smtpErrorAddresses;
my $emHost='localhost';
my $emPort=5001;
my $emUser='Guest';
my $emPassword='Guest';
my $java='java';
my $clwJar='CLWorkstation.jar';
my $startDate='1 week ago';
my $startDateParsed='';
my $endDate='today at 0:00';
my $endDateParsed='';
my $formatCommandLine='pdf';
my @format;
my $help='';
my $smtpHost='localhost';
my $smtpFrom='reports@introscope.com';



my @listOfReports;

eval {
	if (! GetOptions (
		"emHost=s" => \$emHost,
		"emPort=i" => \$emPort,
		"emUser=s" => \$emUser,
		"emPassword=s" => \$emPassword,
		"java=s" => \$java,
		"clwJar=s" => \$clwJar,
		"reportName=s" => \$reportName,
		"managementModule=s" => \$managementModule,
		"help" => \$help,
		"start=s" => \$startDate,
		"end=s" => \$endDate,
		"format=s" => \$formatCommandLine,
		"smtpHost=s"=> \$smtpHost,
		"smtpTo=s" => \$smtpTo,
		"smtpFrom=s" => \$smtpFrom,
		"smtpErrorTo=s" => \$smtpErrorTo) ) {
			die "FATAL: Error(s) while parsing the command line";
	}
	
	if ($help) {
		my $helpText = <<HELP_TEXT
sendReport.pl - Perl Script to generate Introscope reports and send them as email attachments
Example: perl sendReport.pl -managementModule Sample -reportName "Application Capacity Planning" -format csv,pdf -start "1 week ago" -smtpTo "someone\@company.com,someoneelse\@company.com"

Requires the following modules:
- Getopt::Long
- Date::Manip
- Net::SMTP
- MIME::Lite

Options:
-help - print a brief message and exit
-smtpTo - mandatory - the list of email addresses to send the reports to (email address separator: , or ;)
-reportName - mandatory - the name of the report to generate
-managementModule - mandatory - the management module the report belongs to
-emHost - optional - the host of the Introscope Enterprise Manager (default value: localhost)
-emPort - optional - the port for the Introscope Enterprise Manager (default value: 5001)
-emUser - optional - the username to connect to the Introscope Enterprise Manager (default value: Guest)
-emPassword - optional - the password to connect to the Introscope Enterprise Manager (default value: Guest)
-java - optional - the path to a 1.4.2 or above java virtual machine (default value: java)
-clwJar - optional - the patch to the Introscope Command Line Workstation jar file (default value: CLWorkstation.jar)
-start - optional - the start date for the report (default value: 1 week ago)
-end - optional - the end date for the report (default value: today at 0:00)
-format - optional - the format for the generated report (default value: pdf, valid values: html,pdf,xls,text,xml,csv, separator: , or ;)
-smtpHost - optional - the host of the smtp server (default value: localhost)
-smtpFrom - optional - the email address the report is coming from (default value: reports\@introscope.com)
-smtpErrorTo - optional - the list of email addresses to send an email to when there was a error while generating the report (default value: email addresses specified with the -smtpTo command line argument, email address separator: , or ;)

Note about entering dates for the -start and -end command line arguments:
This script uses the Date::Manip module and thus is very flexible with dates. The following dates are valid:
- now
- today at 0:00
- last week
- 3 months ago
- 1st thursday in June 2004
- 10/24/2006
- 12:30 Dec 12th 2005

HELP_TEXT
;
		print $helpText;
		exit;
	}


	if (! $smtpTo) {
		die "FATAL: The command line option -smtpTo is mandatory";
	}
	@smtpAddresses = split /\s*[,;]\s*/,$smtpTo;
	
	if ($smtpErrorTo) {
		@smtpErrorAddresses = split /\s*[,;]\s*/,$smtpErrorTo;
	} else {
		@smtpErrorAddresses = @smtpAddresses;
	}

	if (! $managementModule) {
		die "FATAL: The command line option -managementModule is mandatory";
	}

	if (! -f $clwJar) {
		die "FATAL: Unable to find $clwJar (file containing the Introscope command line workstation)";
	}

	if (! $reportName) {
		die "FATAL: The command line option -reportName is mandatory";
	}

	@format = split /\s*[,;]\s*/,$formatCommandLine;
	foreach (@format) {
		if (! $reportFormats{$_}) {
			$_ = lc;
			die "FATAL: $_ is not a valid format for command lines option -format";
		}
	}

	$startDateParsed=UnixDate($startDate,$clwDateFormat);
	if (! $startDateParsed) {
		die qq{FATAL: Unable to parse argument "$startDate" for command line option -startDate};	
	}

	$endDateParsed=UnixDate($endDate,$clwDateFormat);
	if (! $endDateParsed) {
		die qq{FATAL: Unable to parse argument "$endDate" for command line option -endDate};	
	}

	if (Date_Cmp(ParseDate($startDate),ParseDate($endDate)) >=0) {
		die qq{FATAL: Date for command line option -startDate must be before command line option -endDate};	
	}

	my $javaOutput=`"$java" -version 2>&1`;
	#print "javaOutput: $javaOutput\n";
	if ($javaOutput=~/^\s*java version "((\d+\.\d+)[^"]*)"\s*$/im) {
		# $1 is the full jvm version (for example: 1.4.2_06)
		# $2 is the 2 first version numbers of the jvm (for example: 1.4)
		if ($2 < 1.4) {
			die "FATAL: This script requires a java virtual machine version 1.4 or above (version of the current jvm: $1)";
		}
	} else {
		die "FATAL: Unable to validate that $java is a valid java virtual machine";
	}


	##################################
	#                   Generates the reports                              #
	##################################

	foreach (@format) {
		$_  = lc;
		my $random = int( rand(10001));
		my $reportFile="report_" . $$ . "_$random.$_";
		while (-e $reportFile) {
			$random = int( rand(10001));
			$reportFile="report_$$_$random.$_";
		}
		push @listOfReports,$reportFile;
		my $clwCommandLine = qq{$java -Xms${clwJavaHeap}m -Xmx${clwJavaHeap}m -Dhost="$emHost" -Dport="$emPort" -Duser="$emUser" -Dpassword="$emPassword" -jar "$clwJar" generate report named \\"$reportName\\" in management module named \\"$managementModule\\" starting at \\"$startDateParsed\\" ending at \\"$endDateParsed\\" to $reportFile};
		print qq{INFO: Executing "$clwCommandLine"\n};
		print qq{INFO: Please wait while generating report "$reportName" in format $_...\n};
		
		my $clwOutput = `$clwCommandLine 2>&1`;
		
		if ($clwOutput) {
			# an error happened while executing CLW
			if ($clwOutput =~ /UnknownHostException/i) {
				die qq{FATAL: EM host "$emHost" is unknown};
			} elsif ($clwOutput =~ /ConnectException/i) {
				die qq{FATAL: Unable to connect to EM on host "$emHost" port "$emPort"};
			} elsif ($clwOutput =~ /LoginFailedException/i) {
				die qq{FATAL: Login to EM on host "$emHost" port "$emPort" with user "$emUser" failed};
			} elsif ($clwOutput =~ /Cannot find report/i) {
				die qq{FATAL: Unable to find report "$reportName" in management module "$managementModule"};
			} else {
				die qq{FATAL: Unexpected problem while generating report "$reportName"\n$clwOutput};
			}
		} else {
			print qq{INFO: Generated report "$reportName" in format $_\n};
		}
	}
	
	##################################
	#           Sends the reports as email attachments               #
	##################################
	
	my $mailToHeader = "";
	my $counter = 0;
	foreach (@smtpAddresses) {
		if ($counter == 0) {
			$mailToHeader=$_;
		} else {
			$mailToHeader=$mailToHeader . "," . $_;
		}
		$counter++;
	}
	
	my $msg = MIME::Lite->new(
		From    => $smtpFrom,
		To      => $mailToHeader,
		Subject => $smtpNormalSubject,
		Type    =>'multipart/mixed',
	);

	$msg->attach(
		Type     =>'TEXT',
		Data     =>$smtpNormalMessage
	);
	
	my $attachmentFileName= $reportName;
	$attachmentFileName=~s/\W/_/g;
	
	foreach (@listOfReports) {
		if (/\.(\w+)$/) {
			my $extension = $1;
			$msg->attach(
				Type        => $reportFormats{$extension},
				Path        => $_,
				Filename    =>"$attachmentFileName.$extension",
				Disposition => 'attachment'
			);
		}
	}
	
	$msg->send('smtp',$smtpHost,Timeout=>60,SkipBad => 1);
	
	print qq{INFO: Sent report "$reportName" via email to "$mailToHeader"\n};
};

##################################
#   Delete the temporary files that contain the reports    #
##################################

foreach (@listOfReports) {
	if (-e $_) {
		unlink($_) or print qq{WARN: Unable to delete temporary file $_\n};
	}
}

##################################
#   Error handling                                                          #
#   Attempt to send an email with the error message          #
##################################

if ( $@ ) {
	my $originalError = $@;
	# attempt to send an email to let the owner know that there was a problem while generating the report
	# if sending the email fails, we ignore this problem and die with the original error message.
	my $smtp;
	eval {
		$smtp = Net::SMTP->new($smtpHost,Timeout => 60) or die qq{Unable to connect to smtp host "$smtpHost"};
		$smtp->mail( $smtpFrom );
		$smtp->to( @smtpErrorAddresses,{ SkipBad => 1 } ); 

		# Start the mail  
		$smtp->data(); 

		# Send the header  
		# This address will appear in the message
		my $mailToHeader = "";
		my $counter = 0;
		foreach (@smtpErrorAddresses) {
			if ($counter == 0) {
				$mailToHeader=$_;
			} else {
				$mailToHeader=$mailToHeader . "," . $_;
			}
			$counter++;
		}
		$smtp->datasend("To: $mailToHeader\n"); 
		$smtp->datasend("From: $smtpFrom\n");  
		$smtp->datasend("Subject: $smtpErrorSubject\n");  
		$smtp->datasend("\n"); 
		
		$smtpErrorMessage = $smtpErrorMessage . $originalError;
		# Send the body.  
		$smtp->datasend("$smtpErrorMessage\n\n"); 

		# Send the termination string  
		$smtp->dataend();
	};
	
	if ($smtp) {
		# Close the connection  
		$smtp->quit();
	}
	
	die "$originalError\n";
}